<?php
/*
Plugin Name: WordPress HTTPS
Plugin URI: http://mvied.com/projects/wordpress-https/
Description: Prevents HTTPS pages from loading non-HTTPS elements. Also gives the option to disable WordPress 3.0+ from changing links to HTTPS.
Author: Mike Ems
Version: 1.5
Author URI: http://mvied.com/
*/

/**
* Class for the WordPress plugin WordPress HTTPS
*
* @author  Mike Ems
* @package WordPressHTTPS
* @copyright Copyright 2010
*/

if ( !class_exists('WordPressHTTPS') ) {
 class WordPressHTTPS {

  /**
   * Default options
   *
   * @private array
   */
  private $options_default;

  /**
   * WordPress version
   *
   * @var int
   */
  var $wp_version;
  
  /**
   * Initialize plugin
   *
   * @param none
   * @return void
   */
  function __construct() {
   // Format WordPress version for comparison. 2.9 => 29, 3.0 => 30, etc.
   $this->wp_version = (int)substr(str_replace('.','',get_bloginfo('version')), 0, 2);
   
   // Define default options
   $this->options_default = array(
    'wordpress-https_externalurls'         => 0, // Option to change external URL's to HTTPS
    'wordpress-https_bypass'               => 0, // Bypass option to check if external elements can be loaded via HTTPS
    'wordpress-https_disable_autohttps'    => 0, // Disable the feature in WordPress 3.0+ that makes all links HTTPS if the page is being viewed via HTTPS.
    'wordpress-https_exclusive_https'      => 0
   );

   if ( is_admin() ) {
    // Add admin menus
    add_action('admin_menu', array(&$this, 'menu'));
    
    // Set default options
    foreach ( $this->options_default as $option => $value ) {
     if ( get_option($option) === false ) {
      add_option($option, $value);
     }
    }
   } else {
    // If not admin, enable front-end redirection
    add_action('wp_head', array(&$this, 'redirect'));
   }
   
   // Add 'Force SSL' checkbox to add/edit post pages
   add_action('post_submitbox_misc_actions', array(&$this, 'post_checkbox'));
   add_action('save_post', array(&$this, 'post_save'));
  
   // Output buffering
   add_action('plugins_loaded', array(&$this, 'buffer_start'));
   //add_action('wp_footer', array(&$this, 'buffer_end'));

   // Filter HTTPS from links in WP 3.0+
   if ( get_option('wordpress-https_disable_autohttps') == 1 ) {
    add_filter('page_link', array(&$this, 'replace_https'));
    add_filter('post_link', array(&$this, 'replace_https'));
    add_filter('category_link', array(&$this, 'replace_https'));
    add_filter('get_archives_link', array(&$this, 'replace_https'));
    add_filter('search_link', array(&$this, 'replace_https'));
    add_filter('home_url', array(&$this, 'replace_https'));
    add_filter('bloginfo', array(&$this, 'bloginfo'), 10, 2);
    add_filter('bloginfo_url', array(&$this, 'bloginfo'), 10, 2);
   }
  }

  /**
   * Add 'Force SSL' checkbox to add/edit post pages
   *
   * @param none
   * @return void
   */
  function post_checkbox() {
   global $post;

   wp_nonce_field( plugin_basename(__FILE__), 'wordpress-https' );

   $checked = false;
   if ($post->ID) {
    $checked = get_post_meta($post->ID, 'force_ssl', true);
   }
   echo '<div class="misc-pub-section misc-pub-section-last" style="border-top: 1px solid #EEE;"><label>Force HTTPS: <input type="checkbox" value="1" name="force_ssl" id="force_ssl"'.(($checked) ? ' checked="checked"' : '').' /></label></div>';
  }
  
  /**
   * Save Force SSL option to post or page
   *
   * @param int $post_id
   * @return int $post_id
   */
  function post_save( $post_id ) {
   if (array_key_exists('wordpress-https',$_POST)) {
    if ( !wp_verify_nonce( $_POST['wordpress-https'], plugin_basename(__FILE__) )) {
      return $post_id;
    }

    if ( defined('DOING_AUTOSAVE') && DOING_AUTOSAVE ) {
     return $post_id;
    }

    if ( 'page' == $_POST['post_type'] ) {
     if ( !current_user_can( 'edit_page', $post_id ) ) {
      return $post_id;
     }
    } else {
     if ( !current_user_can( 'edit_post', $post_id ) ) {
      return $post_id;
     }
    }

    $forceSSL = (($_POST['force_ssl'] == 1) ? true : false);
    if ($forceSSL) {
     update_post_meta($post_id, 'force_ssl', 1);
    } else {
     delete_post_meta($post_id, 'force_ssl');
    }

    return $forceSSL;
   }
   return $post_id;
  }

  /**
   * Filters HTTPS urls from bloginfo function
   *
   * @param string $result
   * @param string $show
   * @return string
   */
  function bloginfo($result = '', $show = '') {
   if ($show == 'stylesheet_url' || $show == 'template_url' || $show == 'wpurl' || $show == 'home' || $show == 'siteurl' || $show == 'url') {
    $result = $this->replace_https($result);
   }
   return $result;
  }
  
  /**
   * Add admin panel menu option
   *
   * @param none
   * @return void
   */
  function menu() {
   add_options_page('WordPress HTTPS Settings', 'WordPress HTTPS', 'manage_options', 'wordpress-https', array(&$this, 'settings'));
  }

  /**
   * Settings page in admin panel
   *
   * @param none
   * @return void
   */
  function settings() {
   if ( !current_user_can('manage_options') ) {
    wp_die( __('You do not have sufficient permissions to access this page.') );
   }

   if ( $_SERVER['REQUEST_METHOD'] === 'POST' ) {
    $errors = array();

    foreach ($this->options_default as $key => $default) {
     if (!array_key_exists($key, $_POST)) {
      $_POST[$key] = $default;
     } else {
      if ( $key == 'wordpress-https_bypass' && @ini_get('allow_url_fopen') != 1 ) {
       $errors[] = 'PHP configuration error: allow_url_fopen must be enabled to allow the conversion of external elements to HTTPS.';
       $_POST[$key] = 0;
      } else if ($key == 'wordpress-https_disable_autohttps' && $this->wp_version < 30) {
       $_POST[$key] = 0;
      }
     }
     update_option($key, $_POST[$key]);
    }
   }
?>
 <div class="wrap">
  <div id="icon-options-general" class="icon32"><br /></div>
  <h2>WordPress HTTPS Settings</h2>
<?php
   if ( $_SERVER['REQUEST_METHOD'] === 'POST' ) {
    if ( sizeof( $errors ) > 0 ) {
     echo "  <ul id=\"message\">\n";
     foreach ( $errors as $error ) {
      echo "   <li class=\"error\"><p>".$error."</p></li>\n";
     }
     echo "  </ul>\n";
    } else {
     echo "   <div class=\"updated below-h2 fade\" id=\"message\"><p>Items saved.</p></div>\n";
    }
   }
?>
  <form name="form" action="options-general.php?page=wordpress-https" method="post">
   <?php settings_fields('wordpress-https'); ?>
   <table class="form-table">
    <tr valign="top">
     <th scope="row">External HTTPS Elements</th>
     <td>
      <fieldset>
       <legend class="screen-reader-text"><span>Attempt to automatically convert external elements to HTTPS.</span></legend>
       <label for="wordpress-https_externalurls"><input name="wordpress-https_externalurls" type="checkbox" id="wordpress-https_externalurls" value="1"<?php echo ((get_option('wordpress-https_externalurls')) ? ' checked="checked"' : ''); ?> /> Attempt to automatically convert external elements to HTTPS.</label>
       <p class="description">Warning: This option checks that the external element can be loaded via HTTPS while the page is loading. Depending on the amount of external elements, this could affect the load times of your pages.</p>
      </fieldset>
     </td>
    </tr>
    <tr valign="top">
     <th scope="row">Bypass External Check</th>
     <td>
      <fieldset>
       <legend class="screen-reader-text"><span>Disable the option to check if an external element can be loaded over HTTPS.</span></legend>
       <label for="wordpress-https_bypass"><input name="wordpress-https_bypass" type="checkbox" id="wordpress-https_bypass" value="1"<?php echo ((get_option('wordpress-https_bypass')) ? ' checked="checked"' : ''); ?> /> Disable the option to check if an external element can be loaded over HTTPS.</label>
       <p class="description">Warning: Bypassing the HTTPS check for external elements may cause elements to not load at all. Only enable this option if you know that all external elements can be loaded over HTTPS.</p>
      </fieldset>
     </td>
    </tr>
<?php if ($this->wp_version >= 30) { ?>
    <tr valign="top">
     <th scope="row">Disable Automatic HTTPS</th>
     <td>
      <fieldset>
       <legend class="screen-reader-text"><span>Disable the feature in WordPress 3.0+ that makes all links HTTPS if the page is being viewed via HTTPS.</span></legend>
       <label for="wordpress-https_disable_autohttps"><input name="wordpress-https_disable_autohttps" type="checkbox" id="wordpress-https_disable_autohttps" value="1"<?php echo ((get_option('wordpress-https_disable_autohttps')) ? ' checked="checked"' : ''); ?> /> Disable the feature in WordPress 3.0+ that makes all links HTTPS if the page is being viewed via HTTPS.</label>
       <p class="description">When a page is viewed via HTTPS in WordPress 3.0+, all internal page, category and post links are changed to HTTPS. This option will disable that.</p>
      </fieldset>
     </td>
    </tr>
<?php } ?>
    <tr valign="top">
     <th scope="row">Force SSL Exclusively</th>
     <td>
      <fieldset>
       <legend class="screen-reader-text"><span>Exclusively force SSL on posts and pages with the 'Force SSL' option checked. All others are redirected to HTTP.</span></legend>
       <label for="wordpress-https_exclusive_https"><input name="wordpress-https_exclusive_https" type="checkbox" id="wordpress-https_exclusive_https" value="1"<?php echo ((get_option('wordpress-https_exclusive_https')) ? ' checked="checked"' : ''); ?> /> Exclusively force SSL on posts and pages with the `Force SSL` option checked. All others are redirected to HTTP.</label>
       <p class="description">WordPress HTTPS adds a 'Force SSL' checkbox to each post and page right above the publish button (screenshot). When selected, the post or page will be forced to HTTPS. With this option enabled, all posts and pages without 'Force SSL' checked will be redirected to HTTP.</p>
      </fieldset>
     </td>
    </tr>
   </table>
   <p class="submit"><input type="submit" name="Submit" value="Save Changes" class="button-primary" /></p>
   <p>This plugin was created by <a href="http://mvied.com/" target="_blank">Mike Ems</a>.</p>
   <p>If you found my plugin useful, please <a href="https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=6ZL95VTJ388HG" target="_blank">donate</a>.</p>
   <p>If you need technical assistance, want to provide feedback, or just want to say thanks, <a href="http://wordpress.org/tags/wordpress-https#postform" target="_blank">drop me a line</a>.</p>
  </form>
 </div>
<?php
  }
  
  /**
   * Redirect posts and pages accordingly
   *
   * @param none
   * @return void
   */
  function redirect() {
   global $post;
   if (is_single() || is_page() || is_home()) {
    if ( $_SERVER['SERVER_PORT'] != 443 ) {
     $forceSSL = get_post_meta($post->ID, 'force_ssl');
     if ($forceSSL) {
      header("Location: https://".$_SERVER['SERVER_NAME'].$_SERVER['REQUEST_URI']);
      exit;
     }
    } else if ( get_option('wordpress-https_exclusive_https') == 1 ) {
     $forceSSL = get_post_meta($post->ID, 'force_ssl');
     if (!$forceSSL) {
      header("Location: http://".$_SERVER['SERVER_NAME'].$_SERVER['REQUEST_URI']);
      exit;
     }
    }
   }
  }

  /**
   * Process output buffer
   *
   * @param string
   * @return string
   */
  function process($buffer) {
   if ( $_SERVER['SERVER_PORT'] == 443 ) {
    if ($this->wp_version >= 30) {
     $httpsUrl = $this->replace_http(get_site_url());
    } else {
     $httpsUrl = $this->replace_http(get_bloginfo('home'));
    }
    $httpUrl = $this->replace_https($httpsUrl);

    preg_match_all('/\<(script|link|img).*(http:\/\/[\/-\w\.]+)/im',$buffer,$matches);
    for ($i = 0; $i<=sizeof($matches[0]); $i++) {
     $html = $matches[0][$i];
     $type = $matches[1][$i];
     $url = $matches[2][$i];
     if ( ( $type == 'link' && strpos($html, 'stylesheet') !== false ) || $type == 'script' || $type == 'img' ) {
      if ( strpos($html,$httpUrl) !== false ) {
       $buffer = str_replace($html, $this->replace_http($html), $buffer);
      } else if ( get_option('wordpress-https_externalurls') == 1 ) {
       if ( get_option('wordpress-https_bypass') == 1 ) {
        $buffer = str_replace($html, $this->replace_http($html), $buffer);
       } else if (@file_get_contents($this->replace_http($url))) {
        $buffer = str_replace($html, $this->replace_http($html), $buffer);
       }
      }
     }
    }
   }
   return $buffer;
  }

  /**
   * Start output buffering
   *
   * @param none
   * @return void
   */
  function buffer_start() {
   if ( get_option('wordpress-https_externalurls') == 1 && get_option('wordpress-https_bypass') != 1 ) {
    @ini_set('allow_url_fopen', 1);
   }
   ob_start(array(&$this, 'process'));
  }

  /**
   * End output buffering
   *
   * @param none
   * @return void
   */
  function buffer_end() {
   ob_end_flush();
  }

  /**
   * Replace HTTPS with HTTP
   *
   * @param string
   * @return string
   */
  function replace_https($string) {
   return str_replace('https://', 'http://', $string);
  }

  /**
   * Replace HTTP with HTTPS
   *
   * @param string
   * @return string
   */
  function replace_http($string) {
   return str_replace('http://', 'https://', $string);
  }

 } // End WordPressHTTPS Class
}

if ( class_exists('WordPressHTTPS') ) {
 $wordpress_https = new WordPressHTTPS();
}
?>